
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "base/polygraph.h"

#include "xstd/Clock.h"
#include "xstd/gadgets.h"
#include "base/ILog.h"
#include "base/OLog.h"
#include "base/ErrorRec.h"


ErrorRec::ErrorRec() {
	reset();
}

ErrorRec::ErrorRec(const Error &e): Error(e) {
	reset();
	theText = e.str();
}

void ErrorRec::reset() {
	theCount = 0;
	theRepCount = 0;
	theRepTime = Time();
	theText = 0;
	theNext = 0;
}

bool ErrorRec::needReport() const {
	return 
		theRepCount <= theCount ||
		theRepTime <= TheClock;
}

void ErrorRec::noteReport() {
	theRepCount = Max(1, theCount) + theCount;
	theRepTime = TheClock + Time::Sec(60);
}

void ErrorRec::store(OLog &ol) const {
	ol << theNo << theCount << theRepCount << theRepTime << theText;
}

void ErrorRec::load(ILog &il) {
	reset();
	il >> theNo >> theCount >> theRepCount >> theRepTime >> theText;
}

void ErrorRec::add(const ErrorRec &e) {
	Assert(theNo == e.theNo);
	theCount += e.theCount;
	theRepCount += theRepCount;
	theRepTime = Max(theRepTime, e.theRepTime);
}
