
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#ifndef PG_CLIENT_CONFIGURATION_H
#define PG_CLIENT_CONFIGURATION_H

#include "xstd/Ring.h"

class Goal;
class RndDistr;
class PopModel;
class RobotSym;
class WarmupPlan;

// Client configuration items that can be shared among multiple clients
class CltCfg {
	public:
		CltCfg();
		~CltCfg();

		void configure(const RobotSym *aRobot);

		int originCount() const { return theOrigins.count(); }

		void startWarmup();
		void stopWarmup();

		int selectViserv();
		bool selectProxy(NetAddr &addr);
		bool selectUsername(String &newName);
		Size selectAbortSize(Size clen);

	protected:
		void configureReqTypes();
		void configureReqMethods();
		void configureOrigins();
		void configureProxies();
		void configureUsernames();

		void addOrigName(const NetAddr &oname);
		void cAddServers(int viserv);

	public:
		const RobotSym *theRobot;  // used to identify/share configs

		Array<int> theOrigins;     // idx to viserv
		RndDistr *theOriginSel;    // selects origin hosts
		RndDistr *theReqTypeSel;   // selects request types
		RndDistr *theReqMethodSel; // selects request methods
		PopModel *thePopModel;     // popularity model to select old oids

		Goal *theBusyPeriod;       // sessions's busy period config
		RndDistr *theIdlePeriodDur;// sessions's idle period config
		Time theSessionHeartbitGap;// hearbit interval

		Ring<NetAddr*> theProxies;
		int theProxyCycleCnt;      // to randomize proxy selection

		WarmupPlan *theWarmupPlan;

		NetAddr thePeerHttp;       // caching peer talking HTTP
		NetAddr thePeerIcp;        // caching peer talking ICP
		double thePublicRatio;     // how often to request a "public" object
		double theRecurRatio;      // how often to re-visit an old object
		double theEmbedRecurRatio; // how often to re-visit an old embedded obj
		double theAbortProb;	   // prob of aborting a transaction

		int theWaitXactLmt;        // limit on the number of postponed requests
		int theIcpPort;            // IPC client should use this port

		Ring<String*> theUsernames;
		int theUsernameCycleCnt;

		bool genUniqUrls;          // each generated URL must be unique
		bool didWarmup;            // warmup plan was completed
};


class CltSharedCfgs: protected Array<CltCfg*> {
	public:
		CltCfg *getConfig(const RobotSym *cfg);

	protected:
		CltCfg *addConfig(const RobotSym *cfg);
};

#endif
