
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "base/polygraph.h"

#include "xstd/h/iostream.h"
#include "xstd/h/iomanip.h"

#include "xstd/Clock.h"
#include "xstd/gadgets.h"
#include "runtime/PubWorld.h"
#include "runtime/StatPhaseMgr.h"
#include "runtime/StatPhase.h"
#include "runtime/LogComment.h"
#include "client/WarmupPlan.h"

int WarmupPlan::TheReqCnt = 0;


WarmupPlan::WarmupPlan(const Array<int> &visIdx): thePlan(visIdx.count()) {
	for (int i = 0; i < visIdx.count(); ++i)
		thePlan.enqueue(visIdx[i]);
	static RndGen rng;
	thePlan.randomize(rng);
}

int WarmupPlan::selectViserv() {
	int viserv = -1;

	while (!thePlan.empty() && viserv < 0) {
		const int vs = thePlan.dequeue();

		if (ThePubWorlds[vs]->size())
			continue;

		thePlan.enqueue(vs);
		viserv = vs;

		TheReqCnt++;
	}

	if (thePlan.count())
		ReportProgress();
	else
	if (viserv < 0)
		CheckTermination();

	return viserv;
}

void WarmupPlan::ReportProgress() {
	const Time reportGap = Time::Sec(45); // how often to report
	static const Time start = TheClock;
	static Time nextReport = start + reportGap;

	if (nextReport >= TheClock)
		return;

	const int worldCnt = ThePubWorlds.count();
	const int readyCnt = ReadyCount();
	const double readyPct = Percent(readyCnt, worldCnt);

	Comment(5) << "fyi:"
		<< " servers visible: " << worldCnt
		<< " requests sent: " << TheReqCnt
		<< " servers ready-to-be-hit: "
			<< setw(5) << readyCnt << " ("
			<< setw(6) << readyPct << "%)"
		<< endc;

	nextReport = TheClock + reportGap;
}

void WarmupPlan::CheckTermination() {
	const int worldCnt = ThePubWorlds.count();
	if (ReadyCount() < worldCnt)
		return;

	static bool didOnce = false;
	if (!didOnce) {
		Comment(3) << "fyi: server scan completed with all local robots"
			<< " ready to hit all " << worldCnt << " visible servers" 
			<< endc;

		TheStatPhaseMgr->unlock();
		didOnce = true;
	}
}

int WarmupPlan::ReadyCount() {
	int count = 0;
	for (int i = 0; i < ThePubWorlds.count(); ++i) {
		if (ThePubWorlds[i]->size())
			count++;
	}
	return count;
}

