
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "base/polygraph.h"

#include "xstd/h/iostream.h"
#include "xstd/h/strstream.h"

#include "xstd/gadgets.h"
#include "runtime/IOBuf.h"
#include "base/ObjId.h"
#include "csm/EmbedContMdl.h"
#include "csm/ContainerBodyIter.h"
#include "csm/oid2Url.h"


ContainerBodyIter::ContainerBodyIter() {
	reset();
}

void ContainerBodyIter::embedContModel(EmbedContMdl *aModel) {
	Assert(!theModel && aModel);
	theModel = aModel;
}

void ContainerBodyIter::reset() {
	BodyIter::reset();
	theModel = 0;
	theEmbedGoal = -1;
	theEmbedCount = 0;
	theNewEmbedCount = 0;
	theEmbedDist = -1;
}

void ContainerBodyIter::configure(WrBuf *aBuf, Size aContSize) {
	BodyIter::configure(aBuf, aContSize);
	Assert(theModel);
	Assert(theEmbedCount == 0);
	theEmbedGoal = theModel->embedGoal(theRng);
	Assert(theEmbedGoal >= 0);
	theEmbedDist = theContSize / (theEmbedGoal+1);
}

void ContainerBodyIter::pour() {
	while (theBuiltSize < theContSize && !theBuf->full()) {
		// fill up to next embed tag
		const Size nextSizeGoal = theEmbedCount < theEmbedGoal ?
			Max(theBuiltSize, theEmbedDist*theEmbedCount + theEmbedDist) :
			theContSize;
		if (const Size left = nextSizeGoal - theBuiltSize) {
			theBuiltSize += theBuf->appendRndUpTo(
				theBuf->normalRndOffset(offSeed(), theBuiltSize), left);
		}

		// put the embed tag
		if (theEmbedCount < theEmbedGoal) {
			if (const int sz = embed())
				theBuiltSize += sz;
			else
				break; // not enough buf space to put the tag!
		}
	}
}

// writes embedded oid tag into the buffer if there is enough space
// and we are not exceeding content length; return #bytes appended
Size ContainerBodyIter::embed() {
	// some strstream cannot accept zero-sized buffers
	if (!theBuf->spaceSize())
		return 0;

	const ObjId eid = theModel->embedOid(theRng, theOid, theNewEmbedCount);

	ostrstream os(theBuf->space(), theBuf->spaceSize());
	os << "<embed src=\"";
	Oid2Url(eid, os);
	os << "\">";
	const bool fit = os != 0;
	const Size sz = os.tellp();

	// give up if the tag does not fit into content-length left
	// give up if the tag is bigger than the buffer
	if (sizeLeft() < sz || (!fit && theBuf->empty())) {
		return theBuf->appendRndUpTo(
			theBuf->normalRndOffset(offSeed(), theBuiltSize), sizeLeft());
	}

	if (fit) {
		theBuf->appended(sz);
		Assert(*(theBuf->space()-1) == '>'); // XXX: remove
		theEmbedCount++;
		return sz;
	}

	// the tag will probably fit, given more buffer space
	return 0;
}
