
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "base/polygraph.h"

#include "runtime/Farm.h"
#include "csm/BodyIter.h"
#include "csm/ContentCfg.h"
#include "csm/ContentMgr.h"

#include "pgl/ContentSym.h"

#include "csm/EmbedContMdl.h"
#include "base/rndDistrStat.h"


EmbedContMdl::EmbedContMdl(): theCfgSel(0), theEmbedCount(0), theEmbedCountMean(-1) {
}

void EmbedContMdl::configure(const ContentSym *cfg) {
	Assert(!theEmbedCount);

	if ((theEmbedCount = cfg->embedCount()))
		theEmbedCountMean = RndDistrStat(theEmbedCount).mean();

	configureEmbedCfgs(cfg);

	if (theEmbedCount && !theCfgs.count()) {
		cerr << cfg->loc() 
			<< "the number of embedded objects per container is specified, "
			<< "but object content types are undefined"
			<< endl;
		exit(-2);
	}

	if (!theEmbedCount && theCfgs.count()) {
		cerr << cfg->loc() 
			<< "object content types are specified, "
			<< "but the number of embedded objects per container is not"
			<< endl;
		exit(-2);
	}
}

double EmbedContMdl::compContPerCall(const ContentCfg *cc) const {
	int idx = 0;
	if (find(cc->id(), idx))
		return theEmbedCountMean * theProbs[idx];
	return 0.0;
}

void EmbedContMdl::noteNewContProb(ContentCfg *cc, double newProb) {
	int idx = 0;
	if (!find(cc->id(), idx))
		return;

	theNewContProb[idx] = newProb;
	Assert(cc->newPerOid() > 0);
}

int EmbedContMdl::embedGoal(RndGen &rng) {
	if (!theEmbedCount)
		return 0;
	theEmbedCount->rndGen(&rng); // note: we do not reset rng back
	return (int)theEmbedCount->trial();
}

ObjId EmbedContMdl::embedOid(RndGen &rng, const ObjId &oid, int &newOidCount) {
	Assert(theEmbedCount);

	ObjId eoid = oid;
	eoid.world(oid.world().genMutant());

	// select the embeded content type
	theCfgSel->rndGen(&rng);   // note: we do not reset rng back
	const int idx = (int)theCfgSel->trial();
	Assert(0 <= idx && idx < theCfgs.count());
	const ContentCfg *cc = theCfgs[idx];
	eoid.type(cc->id());

	newOidCount++;
	Assert(newOidCount <= 255); // to simplify oid generation scheme
	eoid.name(abs((oid.name() << 8) | newOidCount));

	return eoid;
}

void EmbedContMdl::configureEmbedCfgs(const ContentSym *cfg) {
	Array<ContentSym*> syms;
	cfg->contains(syms, theCfgSel, theProbs);

	theCfgs.stretch(syms.count());
	for (int i = 0; i < syms.count(); ++i) {
		const ContentCfg *cfg = TheContentMgr.get(syms[i]);
		theCfgs.append(cfg);
	}

	theNewContProb.stretch(syms.count());
	theNewContProb.count(syms.count());
}

bool EmbedContMdl::find(int cfgId, int &idx) const {
	for (idx = 0; idx < theCfgs.count(); ++idx) {
		if (theCfgs[idx]->id() == cfgId)
			return true;
	}
	return false;
}
