
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#ifndef PG_DNS_MANAGER_H
#define PG_DNS_MANAGER_H

#include "xstd/Array.h"
#include "xstd/Socket.h"
#include "xstd/Dns.h"
#include "xstd/FileScanner.h"

class Client;
class CltXact; // DnsMgr treats this as void
class DnsXact;
class DnsResp;
class DnsResolverSym;

// handles cached asynchronous DNS lookups
class DnsMgr: protected Dns, public FileScanUser {
	public:
		DnsMgr(Client *anOwner);
		virtual ~DnsMgr();

		void configure(const DnsResolverSym *cfg);
		void start();
		void stop();

		bool needsLookup(const NetAddr &addr) const;

		void lookup(const NetAddr &addr, CltXact *x);

		void noteReadReady(int fd);
		void noteXactDone(DnsXact *x);

		int logCat() const;

		Socket &socket() { return theSock; } // for DnsXact

	protected:
		void openSocket();
		void noteReply(const DnsResp &m);

	protected:
		Client *theOwner;
		Array<NetAddr*> theServers; // DNS server addresses
		Time theTimeout;            // one for all xactions

		Array<DnsXact*> theXacts;   // pending transactions

		Socket theSock;
		FileScanReserv theReserv;
		bool closeWithLast;
};

#endif
