
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "pgl/pgl.h"

#include "xstd/String.h"
#include "pgl/PglRec.h"
#include "pgl/PglArraySym.h"
#include "pgl/MimeSym.h"
#include "pgl/ObjLifeCycleSym.h"
#include "pgl/ContentSym.h"


ContentSym::ContentSym(): RecSym("Content", new PglRec) {
	theRec->bAdd("string", "kind", 0);
	theRec->bAdd("Mime", "mime", new MimeSym);
	theRec->bAdd("size_distr", "size", 0);
	theRec->bAdd("ObjLifeCycle", "obj_life_cycle", new ObjLifeCycleSym);
	theRec->bAdd("float", "cachable", 0);
	theRec->bAdd("float", "checksum", 0);
	theRec->bAdd("Content[]", "may_contain", 0);
	theRec->bAdd("int_distr", "embedded_obj_cnt", 0);
	theRec->bAdd("int", "choice_space", 0);
	theRec->bAdd("string", "content_db", 0);
	theRec->bAdd("string", "inject_db", 0);
	theRec->bAdd("float", "infect_prob", 0);
	theRec->bAdd("size_distr", "inject_gap", 0);
}

ContentSym::ContentSym(const String &aType, PglRec *aRec): RecSym(aType, aRec) {
}

bool ContentSym::isA(const String &type) const {
	return RecSym::isA(type) || type == "Content";
}

SynSym *ContentSym::dupe(const String &type) const {
	if (isA(type))
		return new ContentSym(this->type(), theRec->clone());
	return RecSym::dupe(type);
}

bool ContentSym::hasEmbed() const {
	SynSymTblItem *conti = 0;
	Assert(theRec->find("may_contain", conti));
	return conti->sym() != 0;
}

String ContentSym::kind() const {
	return getString("kind");
}

MimeSym *ContentSym::mime() const {
	SynSymTblItem *mi = 0;
	Assert(theRec->find("mime", mi));
	Assert(mi->sym());
	return &(MimeSym&)mi->sym()->cast("Mime");
}

RndDistr *ContentSym::size() const {
	return getDistr("size");
}

ObjLifeCycleSym *ContentSym::objLifeCycle() const {
	SynSymTblItem *olci = 0;
	Assert(theRec->find("obj_life_cycle", olci));
	Assert(olci->sym());
	return &(ObjLifeCycleSym&)olci->sym()->cast("ObjLifeCycle");
}

bool ContentSym::cachable(double &ratio) const {
	return getDouble("cachable", ratio);
}

bool ContentSym::checksum(double &ratio) const {
	return getDouble("checksum", ratio);
}

bool ContentSym::contains(Array<ContentSym*> &ccfgs, RndDistr *&selector, Array<double> &cprobs) const {
	SynSymTblItem *conti = 0;
	Assert(theRec->find("may_contain", conti));
	if (!conti->sym())
		return false; // undefined

	ArraySym &a = (ArraySym&)conti->sym()->cast("Array");
	selector = a.makeSelector("container-contents");
	ccfgs.stretch(a.count());
	cprobs.stretch(a.count());
	a.copyProbs(cprobs);
	for (int i = 0; i < a.count(); ++i) {
		ContentSym *csym = &((ContentSym&)a[i]->cast("Content"));
		ccfgs.append(csym);
	}

	return true;
}

RndDistr *ContentSym::embedCount() const {
	return getDistr("embedded_obj_cnt");
}

bool ContentSym::choiceSpace(int &space) const {
	return getInt("choice_space", space);
}

String ContentSym::cdb() const {
	return getString("content_db");
}

String ContentSym::injectDb() const {
	return getString("inject_db");
}

bool ContentSym::infectProb(double &prob) const {
	return getDouble("infect_prob", prob);
}

RndDistr *ContentSym::injectGap() const {
	return getDistr("inject_gap");
}
