
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "pgl/pgl.h"

#include "xstd/Assert.h"
#include "xstd/Size.h"
#include "xstd/Time.h"
#include "xstd/String.h"
#include "xstd/Rnd.h"
#include "pgl/PglQualifSym.h"
#include "pgl/PglDistrSym.h"


// XXX: we are leaking actual distributions as they are never deleted!

DistrSym::DistrSym(const String &aType, RndDistr *aVal): 
	ExpressionSym(aType), theVal(aVal), theQual(0) {
}

DistrSym::~DistrSym() {
	delete theQual;
}

bool DistrSym::isA(const String &type) const {
	return ExpressionSym::isA(type) || type == "Distr";
}

SynSym *DistrSym::dupe(const String &type) const {
	if (isA(type)) {
		DistrSym *d = new DistrSym(this->type(), theVal);
		d->qualifier(theQual, theQfOp);
		return d;
	}
	return ExpressionSym::dupe(type);
}

void DistrSym::qualifier(const QualifSym *aQual, const String &anOp) {
	delete theQual;
	theQual = aQual ? (QualifSym*)aQual->clone() : 0;
	theQfOp = anOp;
}

ostream &DistrSym::print(ostream &os, const String &) const {
	if (theQual)
		theQual->print(os) << theQfOp;

	if (theVal) {
		if (type() == "int_distr")
			theVal->print(os, &IntArgPrinter);
		else
		if (type() == "size_distr")
			theVal->print(os, &SizeArgPrinter);
		else
		if (type() == "time_distr")
			theVal->print(os, &TimeArgPrinter);
		else
			theVal->print(os);
	} else
		os << "<none>";
	return os;
}

void DistrSym::IntArgPrinter(ostream &os, double arg, int) {
	os << (int)arg;
}

void DistrSym::SizeArgPrinter(ostream &os, double arg, int) {
	os << Size((int)arg);
}

void DistrSym::TimeArgPrinter(ostream &os, double arg, int) {
	os << Time::Secd(arg);
}
