
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#ifndef POLYGRAPH_PGL_SEMANTICS_H
#define POLYGRAPH_PGL_SEMANTICS_H

#include "xstd/Array.h"
#include "pgl/PglBoolSym.h"
#include "pgl/PglExprSym.h"
#include "pgl/PglIntSym.h"
#include "pgl/PglNetAddrSym.h"
#include "pgl/PglQualifSym.h"
#include "pgl/PglNumSym.h"
#include "pgl/PglRecSym.h"
#include "pgl/PglSizeSym.h"
#include "pgl/PglStringSym.h"
#include "pgl/PglTimeSym.h"
#include "pgl/AddrSchemeSym.h"
#include "pgl/pglSyms.h"

class PglCtx;
class ListSym;
class ContainerSym;
class ArraySym;
class KnownNamesSym;
class BenchSym;
class AgentSym;
class DistrSym;
class PhaseSym;
class PopDistrSym;
class StatsSampleSym;
class NetPipeSym;
class AddrMapSym;


// encapsulates reduce-actions called from parser
// and keeps track of the current parsing context
class PglSemx {
	public:
		static void Init(); // these two are called from parser
		static void Done(); // to initialize/destroy semantics info

		static void Destroy(); // wipe out all stored values

		static SynSym *mkCode(StatementSeqSym &);

		static SynSym *mkStatementSeq();
		static SynSym *mkStatementSeq(StatementSym &, StatementSeqSym &);

		static SynSym *mkStatement(SimpleStatementSym &, TokenSym &);
		static SynSym *mkStatement(BlockStatementSym &);

		static SynSym *mkSimpleStatement();
		static SynSym *mkSimpleStatement(SynSym &);

		static SynSym *mkDeclStatement(SynSym &);
		static SynSym *mkExprStatement(SynSym &);
		static SynSym *mkBlockStatement(BlockOpenSym &, SynSym &, BlockCloseSym &);

		static SynSym *mkPureDecl(TypeSym &, CtxNameSym &);
		static SynSym *mkInitDecl(PureDeclSym &, SynSym &, ExpressionSym &);

		static SynSym *mkAssignment(KnownCtxNameSym &, SynSym &, ExpressionSym &);
		static SynSym *mkAssignment(TokenSym &, KnownNamesSym &, TokenSym &, TokenSym &, ExpressionSym &);
		static SynSym *mkBlockOpen(TokenSym &);
		static SynSym *mkBlockClose(TokenSym &);

		static SynSym *mkKnownCtxName(CtxNameSym &s);
		static SynSym *mkCtxName(ObjNameSym &);
		static SynSym *mkObjName(TokenSym &);
		static SynSym *mkObjName(TokenSym &, TokenSym &, ObjNameSym &);

		static SynSym *mkExpression(ExpressionSym &);
		static SynSym *mkExpression(ObjNameSym &);
		static SynSym *mkExpression(TokenSym &);
		static SynSym *mkExpression(BlockStatementSym &);
		static SynSym *mkExpression(TypeSym &, BlockStatementSym &);
		static SynSym *mkExpression(TokenSym &, SynSym &s, TokenSym &);
		static SynSym *mkExpression(UnOperSym &op, ExpressionSym &exp);
		static SynSym *mkExpression(ExpressionSym &exp1, TokenSym &op, ExpressionSym &exp2);
		static SynSym *mkExpression(TokenSym &, TokenSym &, ListSym &arr, TokenSym &);

		static SynSym *mkCall(TokenSym &, TokenSym &, ListSym &arr, TokenSym &);

		static SynSym *mkUnOper(TokenSym &);

		static SynSym *mkType(TokenSym &);
		static SynSym *mkType(TokenSym &s, TokenSym &, TokenSym &);

		static SynSym *mkBool(TokenSym &s);
		static SynSym *mkInt(TokenSym &s);
		static SynSym *mkNum(TokenSym &s);
		static SynSym *mkTime(TokenSym &s);
		static SynSym *mkSize(TokenSym &s);
		static SynSym *mkString(TokenSym &s);
		static SynSym *mkAddr(TokenSym &s);
		static SynSym *mkAddrRange(TokenSym &s);
		static SynSym *mkQualif(TokenSym &s);

		static SynSym *mkKnownNames(KnownCtxNameSym &exp);
		static SynSym *mkKnownNames(KnownCtxNameSym &exp, TokenSym &, ExpressionSym &p);
		static SynSym *mkKnownNames(KnownNamesSym &arr, TokenSym &, KnownCtxNameSym &exp);
		static SynSym *mkKnownNames(KnownNamesSym &arr, TokenSym &, KnownCtxNameSym &exp, TokenSym &, ExpressionSym &p);

		static SynSym *mkArray();
		static SynSym *mkArray(ExpressionSym &exp);
		static SynSym *mkArray(ExpressionSym &exp, TokenSym &, ExpressionSym &p);
		static SynSym *mkArray(ArraySym &arr, TokenSym &, ExpressionSym &exp);
		static SynSym *mkArray(ArraySym &arr, TokenSym &, ExpressionSym &exp, TokenSym &, ExpressionSym &p);

		static SynSym *mkList();
		static SynSym *mkList(ExpressionSym &exp);
		static SynSym *mkList(ListSym &arr, TokenSym &, ExpressionSym &exp);

	protected:
		static SynSymTblItem *FindDescrItem(const TokenSym &name);
		static SynSymTblItem *FindDescr(const TokenSym &name);
		static SynSym &FindSym(const TokenSym &name);
		static const TokenLoc &FindLoc(const TokenSym &name);
		static SynSymTblItem *Declare(const String &type, const String &name, const TokenLoc &loc);
		static void Assign(SynSymTblItem *i, const ExpressionSym &e);
		static SynSym *Clone(SynSym &, ExpressionSym &);
		static ArraySym *newArraySymFor(SynSym &sym);
		static void SetDefault(SynSymTblItem *i);
		static bool KnownType(const String &type);

		static SynSym *CalcExtreme(const TokenSym &name, const ListSym &args, int dir);
		static DistrSym *isDistr(const TokenSym &name, ListSym &arr);
		static PopDistrSym *isPopDistr(const TokenSym &sname, ListSym &args);

		static ArraySym *AgentAddrs(const TokenSym &fname, ListSym &args, AddrSchemeSym::AddrCalc calc);
		static void Use(const ListSym &agents, const TokenSym &pname);
		static void Schedule(const ListSym &phases);
		static void NoteSubstitutes(const ListSym &groups);

		static SynSym *Any2Any(const SynSym &s, const String &type);
		static IntSym *Any2Int(const SynSym &s);
		static NumSym *Any2Float(const SynSym &s);
		static void NoCast(const SynSym &s, const char *totype);

	public:
		// active objects
		static Array<AgentSym*> TheAgentsToUse;
		static Array<NetPipeSym*> TheNetPipesToUse;
		static Array<AddrMapSym*> TheAddrMapsToUse;
		static Array<ContainerSym*> TheAddrSubstsToUse;

		// schedules
		static Array<PhaseSym*> TheSchedule;
		static Array<StatsSampleSym*> TheSmplSchedule;

		static BenchSym *TheBench;  // bench configuration
		static Time TheWorkSetLen;  // working set length
		static int TheWorkSetCap;   // working set capacity
		
	protected:
		static PglCtx *TheCtx;    // current context
		static String TheCtxName; // next context name, if any
};

#endif
