
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "pgl/pgl.h"

#include <ctype.h>

#include "xstd/gadgets.h"
#include "pgl/PglStrRange.h"
#include "pgl/PglStrRangeLexer.h"


/* PglStrRangeLexer::Lexem */

PglStrRangeLexer::Lexem::Lexem(): theStart(0), theStop(0) {
}

PglStrRangeLexer::Lexem::Lexem(const char *aStart, const char *aStop):
	theStart(aStart), theStop(aStop) {
}


/* PglStrRangeLexer */

PglStrRangeLexer::PglStrRangeLexer(PglStrRange *anOwner, const String &anImage):
	theLexems(anImage.len()), theOwner(anOwner), theImage(anImage) {
	Assert(theOwner && theImage);
}

bool PglStrRangeLexer::parse() {
	// split into lexems (numbers, words, or special chars)
	for (const char *p = theImage; *p;) {
		const char *start = p;
		const char ch = *start;
		if (isdigit(ch)) {
			do { ++p; } while (isdigit(*p));
		} else
		if (isalpha(ch)) {
			do { ++p; } while (isalpha(*p));
		} else {
			++p;
		}
		theLexems.enqueue(Lexem(start, p));
	}
	return true;
}

char PglStrRangeLexer::charAt(int pos) const {
	if (theLexems.count() <= pos)
		return (char)0;

	return theLexems.top(pos).tag();
}

void PglStrRangeLexer::step() {
	Assert(theLexems.count());
	Lexem &lm = theLexems.top();
	theOwner->addPointBlock(lm.start(), lm.stop());
	skip();
}

void PglStrRangeLexer::skip(int count) {
	while (count--)
		theLexems.dequeue();
}

int PglStrRangeLexer::intAt(int pos) const {
	Assert(pos < theLexems.count());
	int n = -1;
	Assert(isInt(theLexems.top(pos).start(), n));
	return n;
}

// int or int-int
bool PglStrRangeLexer::rangeItem() {
	if (!isdigit(charAt(0)))
		return false;
	const int start = intAt(0);

	if (charAt(+1) == '-') {

		if (!isdigit(charAt(+2)))
			return false;
		const int stop = intAt(+2) + 1;

		if (stop < start)
			return false;

		skip(3);
		theOwner->addRangeBlock(start, stop);
		return true;
	}

	skip(1);
	theOwner->addRangeBlock(start, start+1);
	return true;
}

// rangeItem, rangeItem, ...
bool PglStrRangeLexer::range() {
	bool result = false;
	while (rangeItem()) {
		result = true;
//		if (next() == ',')
//			skip();
	}
	return result;
}
