
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "pgl/pgl.h"

#include "pgl/PglRec.h"
#include "pgl/CacheSym.h"
#include "pgl/RobotSym.h"
#include "pgl/ServerSym.h"
#include "pgl/ProxySym.h"


ProxySym::ProxySym(): AgentSym("Proxy") {
	theRec->bAdd("Robot", "client", 0);
	theRec->bAdd("Server", "server", 0);
	theRec->bAdd("Cache", "cache", new CacheSym);
}

ProxySym::ProxySym(const String &aType, PglRec *aRec): AgentSym(aType, aRec) {
}

bool ProxySym::isA(const String &type) const {
	return AgentSym::isA(type) || type == "Proxy";
}

SynSym *ProxySym::dupe(const String &type) const {
	if (isA(type))
		return new ProxySym(this->type(), theRec->clone());
	return AgentSym::dupe(type);
}

RobotSym *ProxySym::client() const {
	SynSymTblItem *ri = 0;
	Assert(theRec->find("client", ri));
	if (ri->sym())
		return &(RobotSym&)ri->sym()->cast("Robot");
	return 0;
}

ServerSym *ProxySym::server() const {
	SynSymTblItem *si = 0;
	Assert(theRec->find("server", si));
	if (si->sym())
		return &(ServerSym&)si->sym()->cast("Server");
	return 0;
}

CacheSym *ProxySym::cache() const {
	SynSymTblItem *ci = 0;
	Assert(theRec->find("cache", ci));
	if (ci->sym())
		return &(CacheSym&)ci->sym()->cast("Cache");
	cerr << loc() << "must configure cache for each proxy" << endl << xexit;
	return 0;
}

const char *ProxySym::msgTypesField() const {
	Assert(false); // should not be called
	return 0;
}
