
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "base/polygraph.h"

#include "xstd/Rnd.h"
#include "xstd/String.h"
#include "xstd/Socket.h"
#include "runtime/LogComment.h"
#include "runtime/polyBcastChannels.h"
#include "base/RndPermut.h"
#include "runtime/Agent.h"
#include "pgl/AgentSym.h"
#include "pgl/SocketSym.h"

#include "runtime/globals.h"

int Agent::TheLastSeqvId = 0;

Agent::Agent(): theThinkDistr(0), theCache(0) {
	theSeqvId = ++TheLastSeqvId;
}

Agent::~Agent() {
}

void Agent::configure(const AgentSym *cfg, const NetAddr &aHost, SockOpt &opt) {
	Assert(TheFileScanner);
	Assert(cfg);

	theHost = aHost;
	theKind = cfg->kind();

	if (UniqId id = cfg->world())
		theId = id;
	else
		theId.create();

	if (!theThinkDistr) {
		theThinkDistr = cfg->xactThink();
		if (theThinkDistr)
			theThinkDistr->rndGen(LclRndGen("agent_xact_think"));
	}

	// socket options
	bool ng = true;
	if (cfg->socket()->nagle(ng))
		opt.nagle = ng ? 1 : -1;
	cfg->socket()->lingerTout(opt.linger);

	describe(Comment(5) << "created ");
	Comment << endc;
}

void Agent::cache(Cache *aCache) {
	Assert(!theCache);
	theCache = aCache;
}

void Agent::start() {
	Broadcast(TheAgentBegChannel, this);
}

void Agent::stop() {
	Broadcast(TheAgentEndChannel, this);
}

void Agent::describe(ostream &os) const {
	const String label = theKind ? theKind : String("agent");
	os << label << " [" << seqvId() << " / " << theId << "] on " << theHost;
}
