
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "base/polygraph.h"

#include "xstd/h/math.h"

#include "xstd/Assert.h"
#include "xstd/Rnd.h"
#include "xstd/gadgets.h"
#include "pgl/PopDistr.h"
#include "pgl/PopModelSym.h"
#include "runtime/PopModel.h"


PopModel::PopModel(): theDistr(0), theHotSetProb(-1), theHotSetFrac(1) {
}

void PopModel::configure(const PopModelSym *cfg) {
	theDistr = cfg->popDistr();
	cfg->hotSetFrac(theHotSetFrac);
	cfg->hotSetProb(theHotSetProb);

	if (!theDistr) {
		cerr << cfg->loc() << "popularity distribution is required if PopModel is used" << endl;
		exit(-2);
	}
}

int PopModel::choose(RndGen &rng, int lastOid, int wss, int hotSetPos, bool &hot) {
	Assert(wss != 0);
	Assert(lastOid > 0);

	hot = theHotSetProb > 0 && rng.event(theHotSetProb);

	// adjust params if we need a "hot" object
	if (hot) {
		wss = wss > 0 ? Max(1, (int)rint(wss*theHotSetFrac)) : lastOid;
		lastOid = MiniMax(wss, hotSetPos, lastOid);
		
	}

	const int offset = (0 < wss && wss < lastOid) ? lastOid-wss : 0;
	const int oname = offset + theDistr->choose(rng, lastOid - offset);

	Assert(0 < oname && oname <= lastOid);
	return oname;
}

