
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "base/polygraph.h"

#include "xstd/h/iomanip.h"

#include "runtime/StatPhase.h"
#include "runtime/StatPhaseMgr.h"
#include "xstd/Socket.h"
#include "runtime/LogComment.h"
#include "runtime/Notifier.h"
#include "runtime/PolyOLog.h"
#include "runtime/StatCycle.h"

#include "xstd/gadgets.h"
#include "runtime/globals.h"
#include "base/polyLogTags.h"
#include "base/polyLogCats.h"


class StatCycleRec: public StatIntvlRec {
};



StatCycle::StatCycle(): theRecs(lgcEnd), theNotifier(0) {
	for (int i = 0; i < theRecs.capacity(); ++i)
		theRecs.push(new StatCycleRec);
}

StatCycle::~StatCycle() {
	while (theRecs.count()) delete theRecs.pop();
}

void StatCycle::period(Time aPeriod) {
	thePeriod = aPeriod;
}

void StatCycle::start() {
	Assert(thePeriod > 0);
	startListen();
	nextIntvl(TheClock + thePeriod);
}

void StatCycle::notifier(Notifier *aNotifier) {
	theNotifier = aNotifier;
}

void StatCycle::wakeUp(const Alarm &alarm) {
	StatIntvl::wakeUp(alarm);
	restart();
	nextIntvl(TheClock + thePeriod); // clock skew is possible
}

void StatCycle::nextIntvl(Time wakeUpTime) {
	theIntvlStart = TheClock;
	sleepTill(wakeUpTime);
}

void StatCycle::restart() {
	Assert(theIntvlStart > 0);
	setDuration(theIntvlStart);
	storeAll(TheOLog, lgStatCycleRec);
	if (TheReportCat > 0)
		report(*theRecs[TheReportCat]);
	for (int i = 0; i < theRecs.count(); ++i) {
		StatCycleRec &rec = *theRecs[i];
		if (theNotifier && IsActiveCat[i])
			theNotifier->noteStats(rec, i);
		rec.restart();
	}
}

StatIntvlRec &StatCycle::getRec(int cat) {
	Assert(0 < cat && cat < theRecs.count());
	return *theRecs[cat];
}

const StatIntvlRec &StatCycle::getRec(int cat) const {
	Assert(0 < cat && cat < theRecs.count());
	return *theRecs[cat];
}

void StatCycle::report(const StatCycleRec &rec) const {
	// note: using Comment leads to part of this info being logged twice
	// once as ascii and once as binary
	Comment(2) << "i-" << TheStatPhaseMgr->name();
	Comment	<< ' ' << setw(6) << TheProgress.xacts();
	rec.linePrint(Comment);
	Comment << ' ' << setw(4) << Socket::Level()
		<< endc;
}
